PetBattleTeams = CreateFrame("frame")
local START = 1
local FINISHED = 5
local PETS_PER_TEAM = 3
local MAX_TEAMS = 12


local function TooltipSetUnit (self, petID, team, petIndex)
	
	local speciesID, customName, level, xp, maxXp, displayID, petName, petIcon, petType, creatureID = C_PetJournal.GetPetInfoByPetID(petID)
	
	
	
	local pet = PetBattleTeamsSettings.teams[team][petIndex]
	
	local health, maxHealth, attack, speed, rarity = C_PetJournal.GetPetStats(petID)
	local r, g, b,hex = GetItemQualityColor(rarity-1)
	
	self.rarityGlow:SetVertexColor(r, g, b)
	self.Icon:SetTexture(petIcon)
	self.Name:SetText("|c"..hex..petName.."|r")
	self.Level:SetText(level)
	self.XPBar:SetWidth(max((xp / max(maxXp,1)) * self.xpBarWidth, 1))
	self.Delimiter:SetPoint("TOP", self.XPBG, "BOTTOM", 0, -10)
	self.XPText:SetFormattedText(self.xpTextFormat or PET_BATTLE_CURRENT_XP_FORMAT, xp, maxXp)
	self.teamText:SetText("Team "..team)
	self.AttackAmount:SetText(attack)
	self.SpeedAmount:SetText(speed)
	self.PetType.Icon:SetTexture("Interface\\PetBattles\\PetIcon-"..PET_TYPE_SUFFIX[petType])
	
	self.SpeciesName:Hide()
	if customName then
		if ( customName ~= petName ) then
			self.Name:SetText(customName)
			self.SpeciesName:SetText(petName)
			self.SpeciesName:Show()
		end
	end
	
	if ( self.HealthText ) then
		self.HealthText:SetFormattedText(self.healthTextFormat or PET_BATTLE_CURRENT_HEALTH_FORMAT, health, maxHealth)
	end
	
	if ( health == 0 ) then
		self.ActualHealthBar:SetWidth(1)
	else
		self.ActualHealthBar:SetWidth((health / max(maxHealth,1)) * self.healthBarWidth)
	end
		
	for i=1, #pet.abilities do
		local name, icon, petType = C_PetJournal.GetPetAbilityInfo(pet.abilities[i])
		local abilityIcon = self["AbilityIcon"..i]
		local abilityName = self["AbilityName"..i]

		abilityName:SetShown(name)
		abilityIcon:SetShown(icon)
		abilityIcon:SetTexture("Interface\\PetBattles\\BattleBar-AbilityBadge-Neutral")
		if name then 
			abilityName:SetText(name)
		end
	end
end
--update functions

local function UpdateAddTeamButton(self,numTeams)
	if self then
		self:Show()
		
		if numTeams == 0 then
			self:SetPoint("TOPLEFT", self:GetParent() ,"TOPLEFT",17,-30)
		else
			self:SetPoint("TOPLEFT","PetTeam"..numTeams.."Button"..1,"BOTTOMLEFT",-1,-3)
		end
		
		if numTeams == MAX_TEAMS then 
			
			self.icon:SetDesaturated(true)
			
		else
			
			self.icon:SetDesaturated(false)
		end
		
		
		
	end
end

local function UpdateReviveButton(self,numTeams)
	if self then
		if not PetBattleTeamsSettings.attached  then
			self:Show();
			if numTeams == 0 then
				self:SetPoint("TOPRIGHT", self:GetParent() ,"TOPRIGHT",-17,-30)
				self:Show()
			else 
				self:SetPoint("TOPRIGHT","PetTeam"..numTeams.."Button"..3,"BOTTOMRIGHT",0,-3)
				self:Show()
			end
		else
			self:Hide();
		end
	end
end

local function UpdateBgFrame(self, numTeams)
	if self then
	
		if PetBattleTeamsSettings.attached and PetJournal then
			PetBattleTeamsSettings.lockBgFrameLocation = false;
			self:SetParent(PetJournal)
			self:ClearAllPoints();
			self:SetPoint("TOPLEFT",PetJournal,"TOPRIGHT",-5,4)
		else
			self:SetParent("UIParent")
		end
		
		if PetBattleTeamsSettings.lockBgFrameLocation or PetBattleTeamsSettings.attached then 
			self:SetMovable(false)
			self:SetScript("OnDragStart", nil)
			self:SetScript("OnDragStop", nil)
		else
			self:SetScript("OnDragStart", self.StartMoving)
			self:SetScript("OnDragStop", self.StopMovingOrSizing)
			self:SetMovable(true)
			
			
			self.menu:SetMovable(true)
		end
		
		if self then
			if self:GetParent() == PetJournal then
				self:SetSize(165,614)
			else
				
				self:SetSize(165,44.3*(numTeams)+45+35)
				--self:SetSize(165,bbottom-abottom)
				--if #PetBattleTeamsSettings.teams < MAX_TEAMS then
				--	self:SetSize(165,46*(numTeams+1)+35)
				--end
			end
		end
	end
end

local function UpdatePetButton(self)
	self:Hide()
	
	if PetBattleTeamsSettings.teams[self.team] and PetBattleTeamsSettings.teams[self.team][self.petIndex] then
		self:Show()
		
		self.selected:Hide()
		self.Icon:SetTexture("\Interface\\ICONS\\INV_Misc_QuestionMark")
		self.ActualHealthBar:Hide()
		self.rarityGlow:Hide()
		self.BorderAlive:Show()
		self.level:Hide()
		self.BorderDead:Hide()
		self.petID = PetBattleTeamsSettings.teams[self.team][self.petIndex].petID
		if self.petIndex == PETS_PER_TEAM and PetBattleTeamsSettings.selected == self.team then
			
				self.selected:Show()
		end
		
		if self.petID and self.petID > 0 and C_PetJournal.GetPetInfoByPetID(self.petID) then

			-- get info about the pet
			local speciesID, customName, level, xp, maxXp, displayID, name, icon, petType, creatureID, sourceText, description, isWild, canBattle, tradable = C_PetJournal.GetPetInfoByPetID(self.petID)
			local health, maxHealth, attack, speed, rarity = C_PetJournal.GetPetStats(self.petID)
			
			if rarity then
				local r, g, b = GetItemQualityColor(rarity-1)
				self.rarityGlow:SetVertexColor(r, g, b)
			end
			
			--self.level:Hide()
			--if  PetBattleTeamsSettings.displayLevel then
			self.level:SetText(level)
			self.level:Show()
			--end
			
			self.Icon:SetTexture(icon)

			--set health bar
			if health and maxHealth then
				if  ( health == 0 ) then
					self.ActualHealthBar:Hide()
				else
					self.ActualHealthBar:Show()
				end
				self.ActualHealthBar:SetWidth((health / max(maxHealth,1)) * self.healthBarWidth)
			end
			
			--Set alive border
			if health and ( health == 0 ) then
				self.BorderAlive:Hide()
				self.rarityGlow:Hide()
			else
				self.BorderAlive:Show()
				self.rarityGlow:Show()
			end
			
			--set dead border
			if health and ( health == 0 ) then
				self.BorderDead:Show()
				self.rarityGlow:Hide()
			else
				self.BorderDead:Hide()
				self.rarityGlow:Show()
			end
			
			for _, object in pairs(self.hideWhenDeadList) do
				if ( health == 0 ) then
					object:Hide()
				else
					object:Show()
				end
			end
		end
	end
	
end

--Create UI functions
local function DropDownOnLoad(self,level)
	local info = UIDropDownMenu_CreateInfo()
	info.isNotRadio = true
	
	info.text = "Attach to Pet Journal"
	info.func = function(_, _, _, value) 
		PetBattleTeamsSettings.attached = not value
		if PetBattleTeamsSettings.attached then
			PetBattleTeamsSettings.lockBgFrameLocation = false;
		end
		PetBattleTeams:UpdateAllFrames()
	end
	info.checked = function() return PetBattleTeamsSettings.attached end
	UIDropDownMenu_AddButton(info, level)
	
	info.text = "Lock Location"
	info.func = function(_, _, _, value)  
		PetBattleTeamsSettings.lockBgFrameLocation = not value 
		PetBattleTeams:UpdateAllFrames()
	end
	info.checked = function()  return PetBattleTeamsSettings.lockBgFrameLocation end
	UIDropDownMenu_AddButton(info, level)
	
	--[[info.text = "Display Level"
	info.func = function(_, _, _,value) 
		PetBattleTeamsSettings.displayLevel = not value 
		PetBattleTeams:UpdateAllFrames()  
	end
	info.checked = function() return PetBattleTeamsSettings.displayLevel   end
	UIDropDownMenu_AddButton(info, level)--]]
	
	info.notCheckable = true;
	info.text = "Close Menu"
	info.func = nil
	UIDropDownMenu_AddButton(info, level)
end

--Creates fake of blizzards PetBattleUnitTooltip
--Returns a tooltip

 function CreateReviveButton(name,parent)
	local HEAL_PET_SPELL = 125439;
	local spellName, spellSubname, spellIcon = GetSpellInfo(HEAL_PET_SPELL);
	local start, duration, enable = GetSpellCooldown(HEAL_PET_SPELL);
	
	button = CreateFrame("BUTTON",name,parent,"secureactionbuttontemplate")
	button:SetAttribute("type", "spell");
	button.spellID = HEAL_PET_SPELL;
	button:SetAttribute("spell",spellName) --"Revive Battle Pets");
	button:SetSize(40,40);
	button.Icon = button:CreateTexture(name.."Icon","ARTWORK");
	button.Icon:SetTexture(spellIcon);
	button.Icon:SetAllPoints();
	
	
	button.Border = button:CreateTexture(name.."Border","OVERLAY","ActionBarFlyoutButton-IconFrame");
	button:SetPushedTexture("Interface\\Buttons\\UI-Quickslot-Depress")
	button:SetHighlightTexture("Interface\\Buttons\\ButtonHilight-Square","ADD");
	
	button.Cooldown = CreateFrame("COOLDOWN", name.."Cooldown",button, "CooldownFrameTemplate");
	CooldownFrame_SetTimer(button.Cooldown, start, duration, enable);
	
	button:RegisterEvent("SPELL_UPDATE_COOLDOWN");
	button:RegisterEvent("SPELLS_CHANGED");
	button:SetScript("OnEvent", function(self,event,...) 
		if ( event == "SPELL_UPDATE_COOLDOWN" ) then
			local start, duration, enable = GetSpellCooldown(self.spellID);
			CooldownFrame_SetTimer(self.Cooldown, start, duration, enable);
			-- Update tooltip
			if ( GameTooltip:GetOwner() == self ) then
				PetJournalHealPetButton_OnEnter(self);
			end
		elseif ( event == "SPELLS_CHANGED" ) then
			--PetJournalHealPetButton_UpdateUsability(self);
		end
	end)
	
	button:SetScript("OnShow", function(self) 
		self:RegisterEvent("SPELL_UPDATE_COOLDOWN");
		--self:RegisterEvent("SPELLS_CHANGED");
		--PetJournalHealPetButton_UpdateUsability(self);
	end)
	button:SetScript("OnHide",function(self) 
		self:UnregisterEvent("SPELL_UPDATE_COOLDOWN");
		--self:UnregisterEvent("SPELLS_CHANGED");
		--PetJournalHealPetButton_UpdateUsability(self);
	end)
	
	button:SetScript("OnEnter", PetJournalHealPetButton_OnEnter )
	button:SetScript("OnLeave",function()  GameTooltip:Hide(); end);
	--temp code
	
	--button:SetPoint("CENTER","UIParent");
	--button:Show();
	
	return button;
	
end

local function CreateTooltip(name)
	local tooltip = CreateFrame("frame",name,nil,"PetBattleUnitTooltipTemplate")
	tooltip:SetHeight(211)
	tooltip.WeakToLabel:Hide()
	tooltip.ResistantToLabel:Hide()
	
	--icon quality glow
	tooltip.rarityGlow = tooltip:CreateTexture("PetBattleTeamTooltipGlow","OVERLAY")
	tooltip.rarityGlow:SetTexture("Interface\\Buttons\\UI-ActionButton-Border")
	tooltip.rarityGlow:SetBlendMode("ADD")
	tooltip.rarityGlow:ClearAllPoints()
	tooltip.rarityGlow:SetDrawLayer("OVERLAY", 0)
	tooltip.rarityGlow:SetWidth(tooltip.Icon:GetWidth() * 1.7)
	tooltip.rarityGlow:SetHeight(tooltip.Icon:GetHeight() * 1.7)
	tooltip.rarityGlow:SetPoint("CENTER", tooltip.Icon, "CENTER", 0, 0)
	
	--team indicator
	tooltip.teamText = tooltip:CreateFontString("PetBattleTeamTooltipTeamText","OVERLAY","GameFontNormalSmall")
	tooltip.teamText:SetJustifyH("CENTER")
	tooltip.teamText:SetText("Team XX")
	tooltip.teamText:SetTextColor(1,1,1)
	tooltip.teamText:SetSize(0,0)
	tooltip.teamText:SetFont("Fonts\\FRIZQT__.TTF",12,"OUTLINE")
	tooltip.teamText:SetPoint("BOTTOMLEFT",tooltip.Name,"TOPLEFT",0,2)

	
	tooltip.AbilitiesLabel:Show()
	tooltip.XPBar:Show()
	tooltip.XPBG:Show()
	tooltip.XPBorder:Show()
	tooltip.XPText:Show()
	tooltip.teamText:Show()
	
	return tooltip
end

local function CreateMenuButton(name,parent)
	local button = CreateFrame("BUTTON",name,parent);
	button.menu = CreateFrame("Frame",name,frame,"UIDropDownMenuTemplate")
	button.menu:SetPoint("TOPRIGHT",button,"BOTTOMLEFT",0,0)
	UIDropDownMenu_Initialize(button.menu, DropDownOnLoad, "MENU")

	button:SetScript("OnClick", function(self,button) 		
			ToggleDropDownMenu(1, nil, self.menu, self,20,7)
	end)

	--button:SetMovable(true)
	button:EnableMouse(true)
	button:SetSize(33,33);
	button:ClearAllPoints();
	--SetbuttonButton(button,PetBattleTeamsSettings.buttonAngle);
	
	button.icon = button:CreateTexture("PetBattleTeambuttonButtonIcon","ARTWORK");
	--button.icon:SetTexture("Interface\\Icons\\INV_Misc_QuestionMark");
	button.icon:SetTexture("Interface\\Icons\\INV_PET_BATTLEPETTRAINING");
	button.icon:SetSize(21,21);
	button.icon:ClearAllPoints();
	button.icon:SetPoint("TOPLEFT",button,"TOPLEFT",7,-6);
	
	button.overlay = button:CreateTexture("PetBattleTeambuttonButtonIcon","OVERLAY");
	button.overlay:SetTexture("Interface\\MiniMap\\MiniMap-TrackingBorder");
	button.overlay:SetSize(56,56);
	button.overlay:ClearAllPoints();
	button.overlay:SetPoint("TOPLEFT",button,"TOPLEFT");
	
	button:SetHighlightTexture("Interface\\MiniMap\\UI-MiniMap-ZoomButton-Highlight","ADD");
	
	button:RegisterForClicks("LeftButtonUp","RightButtonUp")
	--button:RegisterForDrag("LeftButton","RightButton")
	return button;
end

local function CreateParentFrame(name)
	if not name then error("name is nil") end
	
	local frame = CreateFrame("FRAME", name, UIParent)
	
	--considering moving this out of the function
	--frame:SetPoint("TOPLEFT",UIParent,"TOPRIGHT",-5,4)
	
	frame:SetClampedToScreen(true)
	frame:SetFrameStrata("HIGH")
	frame:SetSize(165,597)
	frame:RegisterForDrag("LeftButton")
	frame:EnableMouse(true)
	frame:SetBackdrop({bgFile = "Interface\\Tooltips\\UI-Tooltip-Background", edgeFile = "Interface\\Tooltips\\UI-Tooltip-Border", tile = false, tileSize = 32, edgeSize=32, insets ={left=8,right=8,top=8,bottom=8}})
	frame:SetBackdropColor(.1,.1,.1,1)
	
	
	--frame label
	frame.name = frame:CreateFontString(name.."Title","OVERLAY","GameFontNormal")
	frame.name:SetJustifyH("CENTER")
	frame.name:SetText("Pet Battle Teams")
	frame.name:SetSize(0,0)
	frame.name:SetPoint("CENTER",frame,"TOP",0,-15)
	
	frame.menu = CreateMenuButton(name.."Menu",frame);
	frame.menu:SetParent(frame)
	frame.menu:ClearAllPoints();
	frame.menu:SetPoint("TOPRIGHT",frame,"TOPRIGHT",8,8)

	return frame
end
--Sets the tooltip to the currently moused over unit
--Add team button related functions
local function CreateAddTeamButton (name,parent)
	local button = CreateFrame("BUTTON", name, parent)
	button:SetSize(40,40)
	
	button.icon = button:CreateTexture(name.."Icon","ARTWORK")
	button.icon:SetTexture("Interface\\GuildBankFrame\\UI-GuildBankFrame-NewTab")
	button.icon:SetAllPoints(button)
	button:SetHighlightTexture("Interface\\Buttons\\ButtonHilight-Square","ADD")
	
	button:SetScript("OnEnter",function(self)
		GameTooltip:SetOwner(self, "ANCHOR_BOTTOMRIGHT");
		GameTooltip:ClearLines();
		
		GameTooltip:AddLine("Click to add a new team")
		if #PetBattleTeamsSettings.teams >= MAX_TEAMS then
			
			GameTooltip:AddLine("At maximum number of teams",1,0,0,true)
		end
		GameTooltip:Show();
	end)
	button:SetScript("OnLeave",function(self)
		GameTooltip:Hide();
	end)
	
	button:SetScript("OnClick",function(self) 
		PetBattleTeams:CreateNewTeam()
		PetBattleTeams:UpdateAllFrames()
	end)
	
	
	return button
end

--pet unitframe functions
local function CreatePetButton(team,petIndex,parent)
	
	local parentName = "PetTeam"..team.."Button"..petIndex
	local button = CreateFrame("BUTTON", parentName, parent, "PetBattleMiniUnitFrameAlly")
	
	button.team = team
	button.petIndex = petIndex
	--button.Icon:SetAllPoints(button);
	--fix template sizes
	button:SetSize(40,40)
	button.healthBarWidth = 37
	button.BorderAlive:SetSize(50,50)
	button.BorderDead:SetSize(50,50)
	if not button.rarityGlow then button.rarityGlow = button:CreateTexture(parentName.."Glow","ARTWORK") end

	button.rarityGlow:SetTexture("Interface\\Buttons\\UI-ActionButton-Border")
	button.rarityGlow:SetBlendMode("ADD")
	button.rarityGlow:ClearAllPoints()
	button.rarityGlow:SetDrawLayer("ARTWORK", 0)
	button.rarityGlow:SetWidth(button:GetWidth() * 1.6)
	button.rarityGlow:SetHeight(button:GetHeight() * 1.4)
	button.rarityGlow:SetPoint("CENTER", button.Icon, "CENTER", 0, 4)
	
	--[[button.levelBubble = button:CreateTexture(parentName.."LevelBubble","OVERLAY")
	button.levelBubble:SetTexture("Interface\\PetBattles\\PetJournal")
	button.levelBubble:SetSize(21,21)
	button.levelBubble:SetTexCoord(0.06835938,0.10937500,0.02246094,0.04296875)
	button.levelBubble:SetPoint("CENTER",button,"TOPLEFT",8,-8)
	--]]
	button.level = button:CreateFontString(parentName.."Level","OVERLAY","GameFontNormalSmall")
	button.level:SetJustifyH("CENTER")
	button.level:SetText("XX")
	button.level:SetSize(0,0)
	button.level:SetFont("Fonts\\FRIZQT__.TTF",11,"OUTLINE")
	button.level:SetPoint("BOTTOMLEFT",button.Icon,"BOTTOMLEFT",2,10)
	
	button.selected = button:CreateTexture(parentName.."Selected","OVERLAY") 
	button.selected:SetSize(36,36)
	button.selected:SetTexture("Interface\\PetBattles\\PetJournal")
	button.selected:SetTexCoord(0.11328125,0.16210938,0.02246094,0.04687500)
	button.selected:SetPoint("CENTER",button,"RIGHT",0,0)
	button.selected:Hide()
	
	button:SetScript("OnLoad",nil)
	button:RegisterForClicks("LeftButtonUp", "RightButtonUp")
	button:SetScript("OnClick", function(self,button) 
		local info, petID = GetCursorInfo();
		
		
		if ( button == "RightButton" ) then
				StaticPopup_Show("PBT_TEAM_DELETE",self.team,nil,self.team)
		elseif button == "LeftButton" then
			if info and info == "battlepet" then
				if PetBattleTeams.CursorStruct then
					
					PetBattleTeams:UpdateTeamSwapPets(self.team,self.petIndex, PetBattleTeams.CursorStruct.sourceTeam,PetBattleTeams.CursorStruct.soucePetIndex);
				else
					PetBattleTeams:UpdateTeamNewPet(petID,self.team,self.petIndex) -- UpdateTeamNewPet =function(self,petID,team,petIndex)
				end
				ClearCursor()
				PetBattleTeams.CursorStruct = nil;
			else
				PetBattleTeams:SetTeam(self.team)
			end
			
			if IsShiftKeyDown() then
				local petLink = C_PetJournal.GetBattlePetLink(self.petID);
				ChatEdit_InsertLink(petLink);
			end
		end
	end)
	
	button:SetScript("OnReceiveDrag", function(self,button) 
		local info, petID = GetCursorInfo();
		if info and info == "battlepet" then
			if PetBattleTeams.CursorStruct then
				
				PetBattleTeams:UpdateTeamSwapPets(self.team,self.petIndex, PetBattleTeams.CursorStruct.sourceTeam,PetBattleTeams.CursorStruct.soucePetIndex);
			else
				PetBattleTeams:UpdateTeamNewPet(petID,self.team,self.petIndex) -- UpdateTeamNewPet =function(self,petID,team,petIndex)
			end
			ClearCursor()
			PetBattleTeams.CursorStruct = nil;
		end
	end)
	
	button:RegisterForDrag("LeftButton")
	button:SetScript("OnDragStart", function(self,button)
		PetBattleTeams.CursorStruct = {}
		PetBattleTeams.CursorStruct.sourceTeam = self.team;
		PetBattleTeams.CursorStruct.soucePetIndex = self.petIndex;
		C_PetJournal.PickupPet(self.petID, false);
	end)

	button:SetScript("OnEnter",function(self) 
		if C_PetJournal.GetPetInfoByPetID(self.petID) then
			
			PetBattleTeams.tooltip:SetParent(self);
			PetBattleTeams.tooltip:SetFrameStrata("TOOLTIP");
			PetBattleTeams.tooltip:ClearAllPoints();
			
			PetBattleTeams.tooltip:SetPoint("TOPLEFT", self, "BOTTOMRIGHT", 0, 0)
			local left, bottom, width, height = PetBattleTeams.tooltip:GetBoundsRect()
			
			if left + width > GetScreenWidth() then 
				PetBattleTeams.tooltip:ClearAllPoints();
				PetBattleTeams.tooltip:SetPoint( "TOPRIGHT", self, "BOTTOMLEFT", 0, 0)
			end
			if bottom < 0 then
				PetBattleTeams.tooltip:ClearAllPoints();
				PetBattleTeams.tooltip:SetPoint("BOTTOMRIGHT", self, "TOPLEFT", 0, 0)
			end

			TooltipSetUnit(PetBattleTeams.tooltip, self.petID, self.team, self.petIndex)
			PetBattleTeams.tooltip:Show()
		end
	end)
	
	button:SetScript("OnLeave",function(self)
		if ( PetBattleTeams.tooltip:GetParent() == self ) then
			PetBattleTeams.tooltip:Hide()
		end
	end)
	button:UnregisterAllEvents()
	return button
end

local function CreatePetUnitFrames(parent)
	local unitFrames = {}
	for team=1,MAX_TEAMS do 
		local buttons = {}
		for petIndex =1, PETS_PER_TEAM do 
			button = CreatePetButton(team,petIndex,parent)
			
			if petIndex==1 and team == 1 then
				
				button:SetPoint("TOPLEFT", parent ,"TOPLEFT",17,-30)
				--button:SetPoint("TOPLEFT", parent ,"TOPLEFT",17,)
			elseif petIndex == 1 and team > 1 then
				button:SetPoint("TOPLEFT","PetTeam"..(team-1).."Button"..petIndex,"BOTTOMLEFT",0,-4)
			else
				button:SetPoint("LEFT","PetTeam"..team.."Button"..petIndex-1,"RIGHT",5,0)
			end
			
			table.insert(buttons,button)
		end
		table.insert(unitFrames,buttons)
	end
	
	return unitFrames
end

--high level team functions
PetBattleTeams.UpdateAllFrames = function(self)
	for i=1, MAX_TEAMS do
		self:UpdatePetTeam(i)
	end
	UpdateAddTeamButton(self.addTeamButton,#PetBattleTeamsSettings.teams)
	UpdateBgFrame(self.bgFrame,#PetBattleTeamsSettings.teams)
	UpdateReviveButton(self.reviveButton, #PetBattleTeamsSettings.teams);
end

PetBattleTeams.UpdatePetTeam = function(self,team)
	if self.petUnitFrames then
		for i=1,PETS_PER_TEAM do
			button = self.petUnitFrames[team][i]
			UpdatePetButton(button)
		end
	end
end

PetBattleTeams.RemoveTeam = function(self,team)
	
	table.remove(PetBattleTeamsSettings.teams,team)
	
	if #PetBattleTeamsSettings.teams > 0 then
		PetBattleTeams:SetTeam(1)
	end

	if #PetBattleTeamsSettings.teams < 14 then
			self.addTeamButton:Show()
	end
end

PetBattleTeams.CreateNewTeam = function(self) 
	if #PetBattleTeamsSettings.teams < MAX_TEAMS then
		local team = {}
		for i = 1,PETS_PER_TEAM do
			pet = {}
			pet.abilities = {}
			pet.petID, pet.abilities[1], pet.abilities[2], pet.abilities[3] = C_PetJournal.GetPetLoadOutInfo(i)
			
				table.insert(team,pet)
			
		end
		if #team > 0 then
			table.insert(PetBattleTeamsSettings.teams,team)
			PetBattleTeamsSettings.selected = #PetBattleTeamsSettings.teams
			if #PetBattleTeamsSettings.teams == MAX_TEAMS then
					self.addTeamButton:Hide()
			end
		end
		
	end
end

PetBattleTeams.UpdateCurrentTeam =function()
	if not PetBattleTeams:IsWorking() then
		local team = {}
		for i = 1,PETS_PER_TEAM do
			pet = {}
			pet.abilities = {}
			pet.petID, pet.abilities[1], pet.abilities[2], pet.abilities[3] = C_PetJournal.GetPetLoadOutInfo(i)
			table.insert(team,pet)
		end
		PetBattleTeamsSettings.teams[PetBattleTeamsSettings.selected] = team
		PetBattleTeams:UpdateAllFrames()
	end
end

PetBattleTeams.UpdateTeamSwapPets=function(self,destinationTeam,destinationPetIndex,sourceTeam,sourcePetIndex)
	
	
	if not PetBattleTeams:IsWorking() then
		petA = PetBattleTeamsSettings.teams[destinationTeam][destinationPetIndex]
		petB = PetBattleTeamsSettings.teams[sourceTeam][sourcePetIndex]
		
		if petA.petID ~= petB.petID and sourceTeam ~= destinationTeam  then
			for i=1,PETS_PER_TEAM do
				if petA.petID == PetBattleTeamsSettings.teams[sourceTeam][i].petID or petB.petID == PetBattleTeamsSettings.teams[destinationTeam][i].petID then
					return
				end
			end
		end
		PetBattleTeamsSettings.teams[destinationTeam][destinationPetIndex] = petB
		PetBattleTeamsSettings.teams[sourceTeam][sourcePetIndex] = petA
		
		
		
		
		if destinationTeam == PetBattleTeamsSettings.selected or sourceTeam == PetBattleTeamsSettings.selected  then 
			self:SetTeam(PetBattleTeamsSettings.selected)
		end
		PetBattleTeams:UpdateAllFrames()
	end
end

PetBattleTeams.UpdateTeamNewPet =function(self,petID,team,petIndex)
	if not PetBattleTeams:IsWorking() then
		if PetBattleTeamsSettings.teams[team][petIndex].petID ~= petID then
			for i=1,PETS_PER_TEAM do
				if petID == PetBattleTeamsSettings.teams[team][i].petID then
					return
				end
			end
		end
		
		local speciesID, customName, level, xp, maxXp, displayID, name, icon, petType, creatureID, sourceText, description, isWild, canBattle, tradable = C_PetJournal.GetPetInfoByPetID(petID)
		if speciesID and canBattle then
			local pet = {}
			local abilities = {}
			C_PetJournal.GetPetAbilityList(speciesID, abilities, {})
			pet.petID = petID
			pet.abilities = {}
			for i=1,3 do
				pet.abilities[i] = abilities[i]
			end
			PetBattleTeamsSettings.teams[team][petIndex] = pet;
			
			if team == PetBattleTeamsSettings.selected  then 
				self:SetTeam(PetBattleTeamsSettings.selected)
			end
			PetBattleTeams:UpdateAllFrames()
		end
	end
end

PetBattleTeams.SetTeam = function(self,team)
	oldTeam = PetBattleTeamsSettings.selected
	PetBattleTeamsSettings.selected = team
	
	PetBattleTeams:UpdateAllFrames()
	PetBattleTeams:StartWork()
end

--Pet switching logic
PetBattleTeams.OnUpdate = function(self)
	if self.step < 4 then    
		if PetBattleTeamsSettings.teams[PetBattleTeamsSettings.selected] and PetBattleTeamsSettings.teams[PetBattleTeamsSettings.selected][self.step] and C_PetJournal.GetPetInfoByPetID(PetBattleTeamsSettings.teams[PetBattleTeamsSettings.selected][self.step].petID) then
			pet = PetBattleTeamsSettings.teams[PetBattleTeamsSettings.selected][self.step]			
			local petID, ability1, ability2, ability3 = C_PetJournal.GetPetLoadOutInfo(self.step)
			if petID ~= pet.petID then 
				C_PetJournal.SetPetLoadOutInfo(self.step,pet.petID) 
				return
			else
				if ability1 ~= pet.abilities[1] then 
					C_PetJournal.SetAbility(self.step, 1, pet.abilities[1])
					return
				elseif ability2 ~= pet.abilities[2] then
					C_PetJournal.SetAbility(self.step, 2, pet.abilities[2])
					return
				elseif ability3 ~= pet.abilities[3] then
					C_PetJournal.SetAbility(self.step, 3, pet.abilities[3])
					return
				end
			end
			
			if petID == pet.petID and ability1 == pet.abilities[1] and ability2 == pet.abilities[2] and ability3 == pet.abilities[3] then
				self.step = self.step +1
			end
		else --invalid pet / non existant
			C_PetJournal.SetPetLoadOutInfo(self.step,0)
			if C_PetJournal.GetPetLoadOutInfo(self.step) == 0 then
				self.step = self.step +1
			end
		end 
	else
		PetJournal_UpdatePetLoadOut()
		self:SetScript("OnUpdate",nil)
		self.step = FINISHED
	end 
end

PetBattleTeams.StartWork = function(self)
	self.step = START
	self:SetScript("OnUpdate",PetBattleTeams.OnUpdate)
end

PetBattleTeams.IsWorking = function(self)
	return self.step < FINISHED 
end

--Reset saved variables
PetBattleTeams.Reset = function(self,resetTeams)
	local backupTeams = {}
	if  PetBattleTeamsSettings then
		backupTeams = PetBattleTeamsSettings.teams
	end
	
	PetBattleTeamsSettings={}
	if resetTeams then
		PetBattleTeamsSettings.teams = {}
		PetBattleTeams:CreateNewTeam()
	else
		PetBattleTeamsSettings.teams = backupTeams;
		if #PetBattleTeamsSettings.teams > MAX_TEAMS then
			for i=#PetBattleTeamsSettings.teams,MAX_TEAMS+1,-1 do
				table.remove(PetBattleTeamsSettings.teams,i);
			end
		end
	end
	
	PetBattleTeamsSettings.version = PetBattleTeams.version
	PetBattleTeamsSettings.selected = 1
	PetBattleTeamsSettings.displayLevel = false 
	PetBattleTeamsSettings.attached = true
	PetBattleTeamsSettings.lockBgFrameLocation = true
	PetBattleTeamsSettings.buttonAngle = 0;
end

-- initialize ui
PetBattleTeams.CreateUI = function(self)	
	if not self.tooltip then self.tooltip = CreateTooltip("PetBattleTeamTooltip")  end
	if not self.bgFrame then self.bgFrame = CreateParentFrame("PetBattleTeamBackgroundFrame") end
	if not self.petUnitFrames then self.petUnitFrames = CreatePetUnitFrames(self.bgFrame) end
	if not self.addTeamButton then self.addTeamButton = CreateAddTeamButton("PetBattleTeamAddTeamButton",self.bgFrame) end
	if not self.reviveButton then 
		self.reviveButton = CreateReviveButton("PetBattleTeamReviveButton",self.bgFrame);
		--self.reviveButton:SetPoint("BOTTOMRIGHT",self.bgFrame,"BOTTOMRIGHT",-17,20);
		self.reviveButton:Show();
	end
end

-- misc functions
PetBattleTeams.OnEvent = function(self,event,...)
    if event == "ADDON_LOADED" then
        name = ...
        if name == "PetBattleTeams" then
			
			local loaded, reason = LoadAddOn("Blizzard_PetJournal")
			
			if not PetBattleTeamsSettings  or PetBattleTeamsSettings.version ~= PetBattleTeams.version then
				self:Reset(self.resetTeams)
				self:UpdateAllFrames()
			end
			self:CreateUI()
			self:UpdateAllFrames()
			self:UnregisterEvent("ADDON_LOADED")
        end
    end
	if event == "PET_JOURNAL_LIST_UPDATE" then 
		self:UpdateAllFrames()
	end
	if event =="PET_JOURNAL_PET_DELETED" then	
		self:UpdateAllFrames()
	end
end

StaticPopupDialogs["PBT_TEAM_DELETE"] = {
	text = "Are you sure you want to delete team |cffffd200%s|r?",
	button1 = OKAY,
	button2 = CANCEL,
	OnAccept = function(self)
		PetBattleTeams:RemoveTeam(self.data)
		PetBattleTeams:UpdateAllFrames()
	end,
	timeout = 0,
	exclusive = 1,
	hideOnEscape = 1,
}

function SlashCmdList.PETBATTLETEAMS(msg, editbox)
	id = tonumber(msg)
	if id and id > 0 and id <= #PetBattleTeamsSettings.teams then
		PetBattleTeams:SetTeam(id)
	elseif msg == "reset" then
		PetBattleTeams:Reset(true)
		PetBattleTeams:UpdateAllFrames()
	elseif msg == "show" then
		PetBattleTeams:Reset(false)
		PetBattleTeams:UpdateAllFrames()
		TogglePetJournal(2)
	else
		print("To switch between pet teams use /pbt # or /petbattleteams #, where # is a number between 1 and ".. #PetBattleTeamsSettings.teams)
		print("To reset your PBT setting use /pbt reset")
		print("To attach or detach the frame from the pet journal ui")
		print("Use /pbt attach and /pbt detach")
	end
end

function PrintObject(object,prepend)
	if not prepend then prepend = "   " end
	
	if object and type(object) == "table" then
		for k,v in pairs(object) do 
			if type(v) == "table" then 
				print(prepend..k) 
				PrintObject(v,prepend.."  ")
			else
				print (prepend..k.." : "..tostring(v))
			end
			
		end
	end
end

--constants
SLASH_PETBATTLETEAMS1, SLASH_PETBATTLETEAMS2 = '/pbt', '/petbattleteams' 

PetBattleTeams.version = 2
PetBattleTeams.resetTeams = false;
PetBattleTeams.step = 5

--Begin addon
PetBattleTeams:RegisterEvent("PET_JOURNAL_PET_DELETED")
PetBattleTeams:RegisterEvent("PET_JOURNAL_LIST_UPDATE")
PetBattleTeams:RegisterEvent("ADDON_LOADED")
PetBattleTeams:SetScript("OnEvent" , PetBattleTeams.OnEvent)

hooksecurefunc(C_PetJournal, "SetPetLoadOutInfo", PetBattleTeams.UpdateCurrentTeam)
hooksecurefunc(C_PetJournal, "SetAbility", PetBattleTeams.UpdateCurrentTeam)